/*
 * Assembly Language Debugger
 *
 * Copyright (C) 2000 Patrick Alken
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: file.c,v 1.4 2001/04/04 21:48:17 cosine Exp $
 */

#include "elfdefs.h"
#include "load.h"
#include "print.h"

/*
PrintHeader()
  Print significant info about the object file's header

Inputs: header - file header

Return: none
*/

void
PrintHeader(void *header)

{
  switch (ObjectFileFormat)
  {
    case OT_ELF:
    {
      PrintElfHeader((Elf32_Ehdr *) header);
      break;
    } /* case OT_ELF */

    default:
    {
      Print(P_COMMAND,
        "%s: Unknown file format",
        ObjectFileName);
      break;
    }
  } /* switch (ObjectFileFormat) */
} /* PrintHeader() */

/*
PrintSectionInfo()
  Print the various sections in a file

Inputs: header - pointer to beginning of file header

Return: none
*/

void
PrintSectionInfo(void *header, char *sname)

{
  switch (ObjectFileFormat)
  {
    case OT_ELF:
    {
      PrintElfSectionInfo((Elf32_Ehdr *) header, sname);
      break;
    } /* case OT_ELF */

    default:
    {
      Print(P_COMMAND,
        "%s: Unknown file format",
        ObjectFileName);
      break;
    }
  } /* switch (ObjectFileFormat) */
} /* PrintSectionInfo() */

/*
PrintSymbols()
  Print the symbols of a file

Inputs: header   - pointer to beginning of file header
        format   - object file format
        filename - object file name

Return: none
*/

void
PrintSymbols(void *header, int format, char *filename)

{
  switch (format)
  {
    case OT_ELF:
    {
      PrintElfSymbols((Elf32_Ehdr *) header);
      break;
    } /* case OT_ELF */

    default:
    {
      Print(P_COMMAND,
        "%s: Unknown file format",
        filename);
      break;
    }
  } /* switch (format) */
} /* PrintSymbols() */

/*
FindSectionByAddr()
  Find the section of the object file which contains the given
address

Inputs: address - file address (virtual)
        start   - modified to contain start of segment
        end     - modified to contain end of segment

Return: 1 if matching segment found
        0 if not
*/

int
FindSectionByAddr(void *header, unsigned int address,
                  unsigned int *start, unsigned int *end)

{
  if (ObjectFileFormat == OT_ELF)
  {
    return (FindElfSectionByAddr((Elf32_Ehdr *) header,
                                 address,
                                 start,
                                 end));
  }

  return (0);
} /* FindSectionByAddr() */

/*
FindSectionByName()
  Find the section of the object file corresponding to the given
name

Inputs: name  - name of section
        start - modified to contain start of segment
        end   - modified to contain end of segment

Return: 1 if matching section found
        0 if not
*/

int
FindSectionByName(void *header, char *name, unsigned int *start,
                  unsigned int *end)

{
  if (ObjectFileFormat == OT_ELF)
  {
    return (FindElfSectionByName((Elf32_Ehdr *) header,
                                 name,
                                 start,
                                 end));
  }

  return (0);
} /* FindSectionByName() */
